@@ -8,7 +8,9 @@
 use bencher::{benchmark_group, benchmark_main, Bencher};
 use hecs::*;
 
+#[derive(Clone)]
 struct Position(f32);
+#[derive(Clone)]
 struct Velocity(f32);
 
 fn spawn_tuple(b: &mut Bencher) {
@@ -195,6 +197,37 @@ fn build(b: &mut Bencher) {
     });
 }
 
+fn build_cloneable(b: &mut Bencher) {
+    let mut world = World::new();
+    let mut builder = EntityBuilderClone::new();
+    builder.add(Position(0.0)).add(Velocity(0.0));
+    let bundle = builder.build();
+    b.iter(|| {
+        world.spawn(&bundle);
+    });
+}
+
+fn access_column(b: &mut Bencher) {
+    let mut world = World::new();
+    let _enta = world.spawn((Position(0.0), Velocity(0.0)));
+    let _entb = world.spawn((true, 12));
+    let entc = world.spawn((Position(3.0),));
+    let _entd = world.spawn((13, true, 4.0));
+    let column = world.column::<Position>();
+    b.iter(|| {
+        let _comp = bencher::black_box(column.get(entc).unwrap());
+    });
+}
+fn spawn_buffered(b: &mut Bencher) {
+    let mut world = World::new();
+    let mut buffer = CommandBuffer::new();
+    let ent = world.reserve_entity();
+    b.iter(|| {
+        buffer.insert(ent, (Position(0.0), Velocity(0.0)));
+        buffer.run_on(&mut world);
+    });
+}
+
 benchmark_group!(
     benches,
     spawn_tuple,
@@ -208,6 +241,9 @@ benchmark_group!(
     iterate_cached_100_by_50,
     iterate_mut_uncached_100_by_50,
     iterate_mut_cached_100_by_50,
-    build
+    build,
+    build_cloneable,
+    access_column,
+    spawn_buffered,
 );
 benchmark_main!(benches);
