@@ -60,6 +60,8 @@ mod archetype;
 mod batch;
 mod borrow;
 mod bundle;
+mod column;
+mod command_buffer;
 mod entities;
 mod entity_builder;
 mod entity_ref;
@@ -69,15 +71,17 @@ mod query_one;
 pub mod serialize;
 mod world;
 
-pub use archetype::Archetype;
+pub use archetype::{Archetype, ArchetypeColumn};
 pub use batch::{BatchIncomplete, BatchWriter, ColumnBatch, ColumnBatchBuilder, ColumnBatchType};
 pub use bundle::{Bundle, DynamicBundle, MissingComponent};
+pub use column::{Column, ColumnMut};
+pub use command_buffer::CommandBuffer;
 pub use entities::{Entity, NoSuchEntity};
-pub use entity_builder::{BuiltEntity, EntityBuilder};
+pub use entity_builder::{BuiltEntity, BuiltEntityClone, EntityBuilder, EntityBuilderClone};
 pub use entity_ref::{EntityRef, Ref, RefMut};
 pub use query::{
-    Access, BatchedIter, PreparedQuery, PreparedQueryBorrow, PreparedQueryIter, Query, QueryBorrow,
-    QueryItem, QueryIter, QueryMut, With, Without,
+    Access, Batch, BatchedIter, Or, PreparedQuery, PreparedQueryBorrow, PreparedQueryIter, Query,
+    QueryBorrow, QueryItem, QueryIter, QueryMut, Satisfies, With, Without,
 };
 pub use query_one::QueryOne;
 pub use world::{
